#!/usr/bin/perl
#
# Copyright (c) 2017 Mathieu Roy <yeupou--gnu.org>
#                   http://yeupou.wordpress.com
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA

use strict;
use feature "switch";  # require for "for when..."
no warnings 'experimental::smartmatch';
use File::Basename;
use File::HomeDir;

my $binmpc = "/usr/bin/mpc";
my $binmpc_nextplaylist = "/usr/local/bin/mpc-playlist-next";
my $binnotify = "/usr/bin/notify-send";
my $bintouch = "/usr/bin/touch";
my $filecurrentplaylist = File::HomeDir->my_home()."/.mpc-current-playlist"; # generated by mpc-playlist-next
my $filemonitorskip = File::HomeDir->my_home()."/.mpc-monitor-skip";

my $notifymessage;

# map of icons, if any
my %icon  = (
    "toggle" =>  "/usr/local/share/utils-mpc-playlist/media-playback-pause.svg.png",
    "next" =>  "/usr/local/share/utils-mpc-playlist/media-skip-forward.svg.png",
    "nextplaylist" =>  "/usr/local/share/utils-mpc-playlist/folder-sync.svg.png",
    "volumeprogressive" =>  "/usr/local/share/utils-mpc-playlist/audio-volume-medium.svg.png",
    "volume" =>  "/usr/local/share/utils-mpc-playlist/audio-volume-medium.svg.png",
    );
# set appropriate icon for command
my $notifyicon = "audio-speakers";
$notifyicon = $icon{@ARGV[0]} if $icon{@ARGV[0]};

# check the command asked for and pass it to mpc (or else)
for (@ARGV[0]) {
    when ("nextplaylist") {
	# specific playlist handling
	system($binmpc_nextplaylist);
    }  
    when ("volumeprogressive") {
	# increase the volume gradually, unless already to the max, otherwise restart at 40 %
	my $volume = "+20";
	# find current volume
	open(MPC_VOLUME, "$binmpc volume |");
	while (<MPC_VOLUME>) {
	    # keep only number, should be the volume, reset to 40 if already reached max
	    chomp(); 
	    s/\D+//g; 
	    $volume = "40" if $_ eq "100"; 
	}
	close(MPC_VOLUME);	
	system($binmpc, "volume", $volume);

	# find out final volume to set in notification
	open(MPC_VOLUME, "$binmpc volume |");
	while (<MPC_VOLUME>) {
	    # keep only number, should be the volume, reset to 40 if already reached max
	    chomp(); 
	    s/\D+//g; 
	    $volume = $_; 
	}
	close(MPC_VOLUME);	
	$notifymessage .= "volume: $volume%\n";
    }      
    default {
	# otherwise, pass the command to mpc as is
	system($binmpc, @ARGV);
    }
}

# find out currently played song and playlist
open(MPC_CURRENT, "$binmpc current |");
while (<MPC_CURRENT>) { chomp(); $notifymessage .= basename($_); }
close(MPC_CURRENT);
if (-r $filecurrentplaylist) {
    open(PLAYLIST_CURRENT, "< $filecurrentplaylist");
    while (<PLAYLIST_CURRENT>) { chomp(); $notifymessage .= "\n\@$_"; }
    close(PLAYLIST_CURRENT);
}

# print a specific message
system($binnotify, "--icon=$notifyicon", $notifymessage);
print $notifymessage;

# set marker so mpc-monitor does not duplicates the mesasge
system($bintouch, $filemonitorskip);

# EOF
